# Import processor classes for easier imports
from datareader.processors.base_processor import BaseProcessor
from datareader.processors.pdf_processor import PDFProcessor
from datareader.processors.pymupdf_processor import PyMuPDFProcessor
from datareader.processors.url_processor import URLProcessor
from datareader.processors.video_processor import VideoProcessor
from datareader.processors.audio_processor import AudioProcessor 