#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

# This setup.py file uses the configuration from setup.cfg
# All configuration parameters are in setup.cfg

if __name__ == '__main__':
    setup() 